@@ -771,7 +771,7 @@ is_gpstime_in_valid_format = function(header, data, behavior = "bool")
   if (is.null(data[["gpstime"]]))
     return(error_handling_engine(errors, behavior))
 
-  if (!header[["Point Data Format ID"]] %in% c(1,3,6,7,8))
+  if (!header[["Point Data Format ID"]] %in% c(1,3,4,5,6,7,8))
     errors = append(errors, "Invalid file: the data contains a 'gpstime' attribute but point data format is not set to 1, 3, 6, 7 or 8.")
 
   return(error_handling_engine(errors, behavior))
